@@ -36,7 +36,7 @@ import java.io.IOException;
  * <p>
  * In the java code each value is represented by an integer.
  * 
- * @version 0.3
+ * @version 0.5
  * @author Michael "Mr. Sump" Poppitz
  *
  */
@@ -47,11 +47,12 @@ public class CapturedData extends Object {
 	 * @param values 32bit values as read from device
 	 * @param triggerPosition position of trigger as index of values array
 	 */
-	protected CapturedData(int[] values, int triggerPosition, int rate, int channels) {
+	protected CapturedData(int[] values, int triggerPosition, int rate, int channels, int enabledChannels) {
 		this.values = values;
 		this.triggerPosition = triggerPosition;
 		this.rate = rate;
 		this.channels = channels;
+		this.enabledChannels = enabledChannels;
 	}
 
 	/**
@@ -61,7 +62,7 @@ public class CapturedData extends Object {
 	 * @throws IOException when reading from file failes
 	 */
 	public CapturedData(File file) throws IOException {
-		int size = 0, r = -1, t = -1, channels = 32;
+		int size = 0, r = -1, t = -1, channels = 32, enabledChannels = -1;
 		String line;
 		BufferedReader br = new BufferedReader(new FileReader(file));
 		do {
@@ -76,6 +77,8 @@ public class CapturedData extends Object {
 				channels = Integer.parseInt(line.substring(11));
 			else if (line.startsWith(";TriggerPosition: "))
 				t = Integer.parseInt(line.substring(18));
+			else if (line.startsWith(";EnabledChannels: "))
+				enabledChannels = Integer.parseInt(line.substring(18));
 		} while (line.startsWith(";"));
 
 		if (size <= 0 || size > 1024 * 256)
@@ -101,6 +104,7 @@ public class CapturedData extends Object {
 		this.triggerPosition = t;
 		this.rate = r;
 		this.channels = channels;
+		this.enabledChannels = enabledChannels;
 
 		br.close();
 	}
@@ -121,11 +125,16 @@ public class CapturedData extends Object {
 			bw.newLine();
 			bw.write(";Channels: " + channels);
 			bw.newLine();
-			bw.write(";TriggerPosition: " + triggerPosition);
+			bw.write(";EnabledChannels: " + enabledChannels);
 			bw.newLine();
+			if (triggerPosition >= 0) {
+				bw.write(";TriggerPosition: " + triggerPosition);
+				bw.newLine();
+			}
 			
 			for (int i = 0; i < values.length; i++) {
-				bw.write(Integer.toHexString(values[i]));
+				String hexVal = Integer.toHexString(values[i]);
+				bw.write("00000000".substring(hexVal.length()) + hexVal);
 				bw.newLine();
 			}
 			bw.close();
@@ -142,4 +151,6 @@ public class CapturedData extends Object {
 	public final int rate;
 	/** number of channels (1-32) */
 	public final int channels;
+	/** bit map of enabled channels */
+	public final int enabledChannels;
 }
