@@ -32,21 +32,38 @@ import java.util.LinkedList;
  * It requires the rxtx package from http://www.rxtx.org/ to
  * access the serial port the analyzer is connected to.
  * 
- * @version 0.3
+ * @version 0.5
  * @author Michael "Mr. Sump" Poppitz
  *
  */
 public class Device extends Object {
-	private final static int SET = 0x80;			// set register command 
-	private final static int TRIGGER = 0x40;		// select trigger registers
-	private final static int RUN = 0x20;			// arm trigger
-	private final static int RESET = 0x10;		// reset
-	private final static int REG0 = 0x02;		// select register 0
-	private final static int REG1 = 0x04;		// select register 1
-	private final static int REG2 = 0x08;		// select register 2
+	/** set trigger mask */
+	private final static int SETTRIGMASK = 0xc0;
+	/** set trigger value */
+	private final static int SETTRIGVAL = 0xc1;
+	/** set clock divider */
+	private final static int SETDIVIDER = 0x80;
+	/** set sample counters */
+	private final static int SETSIZE = 0x81;
+	/** set flags */
+	private final static int SETFLAGS = 0x82;
 
-	private final static int FLAG_DEMUX = 0x00000001;	// demultiplex
-	private final static int FLAG_FILTER = 0x00000002;	// noise filter
+	/** reset analyzer */
+	private final static int RESET = 0x00;
+	/** arm trigger / run device */
+	private final static int RUN = 0x01;
+	/** continue data transmission to host */
+	private final static int XON = 0x11;
+	/** pause data transmission to host */
+	private final static int XOFF = 0x13;
+
+	
+	private final static int FLAG_DEMUX = 0x00000001;		// demultiplex
+	private final static int FLAG_FILTER = 0x00000002;		// noise filter
+	private final static int FLAG_DISABLE_G0 = 0x00000004;	// disable channel group 0
+	private final static int FLAG_DISABLE_G1 = 0x00000008;	// disable channel group 1
+	private final static int FLAG_DISABLE_G2 = 0x00000010;	// disable channel group 2
+	private final static int FLAG_DISABLE_G3 = 0x00000020;	// disable channel group 3
 
 	private final static int CLOCK = 100000000;	// device clock in Hz
 	
@@ -59,10 +76,12 @@ public class Device extends Object {
 		triggerValue = 0;
 		triggerEnabled = false;
 		filterEnabled = false;
-		demux = false;
+		demux = true;
 		divider = 0;
 		stopCounter = 6400;
 		readCounter = 12800;
+		enabledGroups = new boolean[4];
+		setEnabledChannels(-1); // enable all channels
 		
 		percentageDone = -1;
 		
@@ -76,7 +95,7 @@ public class Device extends Object {
 	 */
 	public void setSize(int size) {
 		double ratio = (double)stopCounter / (double)readCounter;
-		readCounter = size - 1;
+		readCounter = size;
 		setRatio(ratio);
 	}
 	
@@ -118,8 +137,15 @@ public class Device extends Object {
 	 * @param value bit map defining what value to wait for on watched channels
 	 */
 	public void setTrigger(int mask, int value) {
-		triggerMask = mask;
-		triggerValue = value;
+		if (!demux) {
+			triggerMask = mask;
+			triggerValue = value;
+		} else {
+			triggerMask = mask & 0xffff;
+			triggerValue = value & 0xffff;
+			triggerMask |= triggerMask << 16;
+			triggerValue |= triggerValue << 16;
+		}
 	}
 	
 	/**
@@ -137,6 +163,18 @@ public class Device extends Object {
 	public void setFilterEnabled(boolean enable) {
 		filterEnabled = enable;
 	}
+	
+	/**
+	 * Set enabled channels.
+	 * @param bit map defining enabled channels
+	 */
+	public void setEnabledChannels(int mask) {
+		enabledChannels = mask;
+		// determine enabled groups
+		for (int i = 0; i < 4; i++)
+			enabledGroups[i] = ((enabledChannels  >> (8 * i)) & 0xff) > 0;
+	}
+
 
 	/**
 	 * Get the maximum sampling rate available.
@@ -161,7 +199,15 @@ public class Device extends Object {
 	public int getTriggerValue() {
 		return (triggerValue);
 	}
-	
+
+	/**
+	 * Returns the currently enabled channels.
+	 * @return bitmask with enabled channels represented as 1
+	 */
+	public int getEnabledChannels() {
+		return (enabledChannels);
+	}
+
 	/**
 	 * Returns wether or not the trigger is enabled.
 	 * @return <code>true</code> when trigger is enabled, <code>false</code> otherwise
@@ -178,6 +224,25 @@ public class Device extends Object {
 		return (filterEnabled);
 	}
 
+	/**
+	 * Returns wether or not the noise filter can be used in the current configuration.
+	 * @return <code>true</code> when noise filter is available, <code>false</code> otherwise
+	 */
+	public boolean isFilterAvailable() {
+		return (!demux);
+	}
+
+	/**
+	 * Returns the number of available channels in current configuration.
+	 * @return number of available channels
+	 */
+	public int getAvailableChannelCount() {
+		if (demux)
+			return (16);
+		else
+			return (32);
+	}
+
 	/**
 	 * Returns wether or not the device is currently running.
 	 * It is running, when another thread is inside the run() method reading data from the serial port.
@@ -200,7 +265,7 @@ public class Device extends Object {
 	 * Gets a string array containing the names all available serial ports.
 	 * @return array containing serial port names
 	 */
-	public String[] getPorts() {
+	static public String[] getPorts() {
 		Enumeration portIdentifiers = CommPortIdentifier.getPortIdentifiers();
 		LinkedList portList = new LinkedList();
 		CommPortIdentifier portId = null;
@@ -258,8 +323,9 @@ public class Device extends Object {
 					SerialPort.STOPBITS_1,
 					SerialPort.PARITY_NONE
 				);
-				port.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
+				port.setFlowControlMode(SerialPort.FLOWCONTROL_XONXOFF_IN);
 				port.disableReceiveFraming();
+				port.enableReceiveTimeout(1000);
 			}
 		} catch(Exception E) {
 			E.printStackTrace(System.out);
@@ -274,12 +340,18 @@ public class Device extends Object {
 	 *
 	 */
 	public void detach() {
-		if (port != null)
-				port.close();
+		if (port != null) {
+			try {
+				// try to make sure device is reset (see run() for loop explanation)
+				for (int i = 0; i < 5; i++)
+					sendCommand(port.getOutputStream(), RESET);
+			} catch (IOException E) {}
+			port.close();
+		}
 	}
 	
 	/**
-	 * Sends a command to the given stream.
+	 * Sends a long command to the given stream.
 	 * 
 	 * @param output	stream to write command to
 	 * @param opcode	one byte operation code
@@ -291,7 +363,7 @@ public class Device extends Object {
 		int mask = 0xff;
 		int shift = 0;
 		
-		raw[0] = (byte)(opcode | 0x01);
+		raw[0] = (byte)opcode;
 		for (int i = 1; i < 5; i++) {
 			raw[i] = (byte)((data & mask) >> shift);
 			mask = mask << 8;
@@ -313,6 +385,31 @@ public class Device extends Object {
 		output.write(raw);
 	}
 	
+	/**
+	 * Sends a short command to the given stream.
+	 * 
+	 * This method is intended to be used for short commands, but can also be called
+	 * with long command opcodes if the data portion is to be set to 0.
+	 * 
+	 * @param output	stream to write command to
+	 * @param opcode	one byte operation code
+	 * @throws IOException if writing to stream fails
+	 */
+	private void sendCommand(OutputStream output, int opcode) throws IOException {
+		byte raw = (byte)opcode;
+		
+		String debugCmd = "";
+		for (int i = 7; i >= 0; i--) {
+			if ((raw & (1 << i)) != 0)
+				debugCmd += "1";
+			else
+				debugCmd += "0";
+		}
+		System.out.println(debugCmd);
+
+		output.write(raw);
+	}
+
 	/**
 	 * Reads <code>channels</code> / 8 bytes from stream and compiles them into a single integer.
 	 * 
@@ -321,11 +418,18 @@ public class Device extends Object {
 	 * @return	integer containing four bytes read
 	 * @throws IOException if stream reading fails
 	 */
-	private int readSample(InputStream input, int channels) throws IOException {
-		int value = 0;
+	private int readSample(InputStream input, int channels) throws IOException, InterruptedException {
+		int v, value = 0;
 
-		for (int i = 0; i < channels / 8; i++){
-			value |= input.read() << (8 * i);
+		for (int i = 0; i < channels / 8; i++) {
+			if (enabledGroups[i]) {
+				v = input.read();
+			} else {
+				v = 0;
+			}
+			if (v < 0 || Thread.interrupted())
+				throw new InterruptedException("Data readout interrupted.");
+			value |= v << (8 * i);
 		}
 
 		return (value);
@@ -337,31 +441,41 @@ public class Device extends Object {
 	 * @return captured data
 	 * @throws IOException when writing to or reading from device fails
 	 */
-	public CapturedData run() throws IOException {
+	public CapturedData run() throws IOException, InterruptedException {
 		OutputStream outputStream = port.getOutputStream();
 		InputStream inputStream = port.getInputStream();
+		// send reset 5 times because in worst case first 4 are interpreted as data of long command
+		for (int i = 0; i < 5; i++)
+			sendCommand(outputStream, RESET);
 
-		sendCommand(outputStream, RESET, 0);
 		if (triggerEnabled) {
-			sendCommand(outputStream, SET | TRIGGER | REG0, triggerMask);
-			sendCommand(outputStream, SET | TRIGGER | REG1, triggerValue);
+			sendCommand(outputStream, SETTRIGMASK, triggerMask);
+			sendCommand(outputStream, SETTRIGVAL, triggerValue);
 		} else {
-			sendCommand(outputStream, SET | TRIGGER | REG0, 0);
-			sendCommand(outputStream, SET | TRIGGER | REG1, 0);
+			sendCommand(outputStream, SETTRIGMASK, 0);
+			sendCommand(outputStream, SETTRIGVAL, 0);
 		}
-		sendCommand(outputStream, SET | REG0, divider);
+		sendCommand(outputStream, SETDIVIDER, divider);
+
+		int flags = 0;
 		if (demux) {
-			sendCommand(outputStream, SET | REG2, FLAG_DEMUX);
-			sendCommand(outputStream, SET | RUN | REG1, ((stopCounter & 0x7fff8) << 13) | ((readCounter & 0x7fff8) >> 3));
+			flags |= FLAG_DEMUX;
+			for (int i = 0; i < 2; i++)
+				if (!enabledGroups[i]) {
+					flags |= FLAG_DISABLE_G0 << i;
+					flags |= FLAG_DISABLE_G2 << i;
+				}
+			sendCommand(outputStream, SETSIZE, (((stopCounter - 8) & 0x7fff8) << 13) | (((readCounter & 0x7fff8) >> 3) - 1));
 		} else {
 			if (filterEnabled)
-				sendCommand(outputStream, SET | REG2, FLAG_FILTER);
-			else 
-				sendCommand(outputStream, SET | REG2, 0);
-			sendCommand(outputStream, SET | RUN | REG1, ((stopCounter & 0x3fffc) << 14) | ((readCounter & 0x3fffc) >> 2));
+				flags |= FLAG_FILTER;
+			for (int i = 0; i < 4; i++)
+				if (!enabledGroups[i])
+					flags |= FLAG_DISABLE_G0 << i;
+			sendCommand(outputStream, SETSIZE, (((stopCounter - 4) & 0x3fffc) << 14) | (((readCounter & 0x3fffc) >> 2) - 1));
 		}
-		// due to the vhdl design the first sample is currently garbage
-		readSample(inputStream, 32);
+		sendCommand(outputStream, SETFLAGS, flags);
+		sendCommand(outputStream, RUN);
 
 		// check if data needs to be multiplexed
 		int channels;
@@ -373,19 +487,34 @@ public class Device extends Object {
 			channels = 32;
 			samples = (readCounter & 0x3fffc);
 		}
+
 		int[] buffer = new int[samples];
-		for (int i = samples - 1; i >= 0; i--) {
-			buffer[i] = readSample(inputStream, channels);
-			percentageDone = 100 - (100 * i) / buffer.length;
+
+		// wait for first byte forever (trigger could cause long delay)
+		for (boolean wait = true; wait == true;) {
+			try {
+				buffer[samples - 1] =  readSample(inputStream, channels);
+				wait = false;
+			} catch (InterruptedException e) {}
 		}
-
-		inputStream.close();
-		outputStream.close();
-		percentageDone = -1;
-
-		int pos = readCounter - stopCounter - 4 / (divider + 1); // 3 cycles for the device to get started
-		int rate = demux?2*CLOCK / (divider + 1):CLOCK / (divider + 1);
-		return (new CapturedData(buffer, pos, rate, channels));
+		
+		// read all other samples
+		try {
+			for (int i = samples - 2; i >= 0 && true; i--) {
+				buffer[i] = readSample(inputStream, channels);
+				percentageDone = 100 - (100 * i) / buffer.length;
+			}
+		} finally {
+			percentageDone = -1;
+			inputStream.close();
+			outputStream.close();
+		}
+		
+		int pos = -1;
+		if (triggerEnabled)
+			pos = readCounter - stopCounter - 2 - (divider == 0 ? 2 : 0);
+		int rate = demux ? 2*CLOCK / (divider + 1) : CLOCK / (divider + 1);
+		return (new CapturedData(buffer, pos, rate, channels, enabledChannels));
 	}
 	
 	private SerialPort port;
@@ -397,6 +526,8 @@ public class Device extends Object {
 	private boolean triggerEnabled;
 	private int triggerMask;
 	private int triggerValue;
+	private int enabledChannels;
+	private boolean enabledGroups[];
 	
 	private int divider;
 	private int stopCounter;
