@@ -42,7 +42,7 @@ import javax.swing.Timer;
  * Its modelled after JFileChooser and should allow for non-dialog implementations
  * making it somewhat reusable.
  * 
- * @version 0.3
+ * @version 0.5
  * @author Michael "Mr. Sump" Poppitz
  *
  */
@@ -85,12 +85,12 @@ public class DeviceController extends JComponent implements ActionListener, Runn
 	 */
 	public DeviceController() {
 		super();
-		setLayout(new GridLayout(16, 2, 5, 5));
+		setLayout(new GridLayout(17, 2, 5, 5));
 		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
 
 		device = new Device();
 
-		String[] ports = device.getPorts();
+		String[] ports = Device.getPorts();
 		portSelect = new JComboBox(ports);
 		add(new JLabel("Analyzer Port:"));
 		add(portSelect);
@@ -105,6 +105,17 @@ public class DeviceController extends JComponent implements ActionListener, Runn
 		speedSelect.addActionListener(this);
 		add(new JLabel("Sampling Rate:"));
 		add(speedSelect);
+
+		Container groups = new Container();
+		groups.setLayout(new GridLayout(1, 4));
+		channelGroup = new JCheckBox[4];
+		for (int i = 0; i < channelGroup.length; i++) {
+			channelGroup[i] = new JCheckBox(Integer.toString(i));
+			channelGroup[i].setSelected(true);
+			groups.add(channelGroup[i]);
+		}
+		add(new JLabel("Channel Groups:"));
+		add(groups);
 		
 		String[] sizes = {
 			"256K", "128K", "64K", "32K", "16K", "8K", "4K",
@@ -115,12 +126,6 @@ public class DeviceController extends JComponent implements ActionListener, Runn
 		add(new JLabel("Recording Size:"));
 		add(sizeSelect);
 
-		String[] ratios = {"0/100", "25/75", "50/50", "75/25", "100/0"};
-		ratioSelect = new JComboBox(ratios);
-		// ratioSelect.setSelectedIndex(2);
-		add(new JLabel("Before/After Ratio: "));
-		add(ratioSelect);
-
 		filterEnable = new JCheckBox("Enable");
 		filterEnable.setSelected(true);
 		filterEnable.setEnabled(false);
@@ -132,6 +137,12 @@ public class DeviceController extends JComponent implements ActionListener, Runn
 		add(new JLabel("Trigger: "));
 		add(triggerEnable);
 		
+		String[] ratios = {"0/100", "25/75", "50/50", "75/25", "100/0"};
+		ratioSelect = new JComboBox(ratios);
+		ratioSelect.setSelectedIndex(2);
+		add(new JLabel("Before/After Ratio: "));
+		add(ratioSelect);
+
 		triggerMask = createChannelList("Trigger Channels (0..31):");
 		triggerValue = createChannelList("Trigger Values (0..31):");
 		
@@ -139,9 +150,9 @@ public class DeviceController extends JComponent implements ActionListener, Runn
 		add(new JLabel("Progress:"));
 		add(progress);
 		
-		JButton go = new JButton("Capture");
-		go.addActionListener(this);
-		add(go);
+		captureButton = new JButton("Capture");
+		captureButton.addActionListener(this);
+		add(captureButton);
 
 		JButton cancel = new JButton("Cancel");
 		cancel.addActionListener(this);
@@ -151,6 +162,7 @@ public class DeviceController extends JComponent implements ActionListener, Runn
 		timer = null;
 		worker = null;
 		status = ABORTED;
+		
 	}
 
 	/**
@@ -172,14 +184,8 @@ public class DeviceController extends JComponent implements ActionListener, Runn
 		// reset progress bar
 		progress.setValue(0);
 
-		// set trigger configuration according to device settings
-		int mask = device.getTriggerMask();
-		int value = device.getTriggerValue();
-		for (int i = 0; i < 32; i++) {
-			triggerMask[i].setSelected(((mask >> i) & 1) != 0);
-			triggerValue[i].setSelected(((value >> i) & 1) != 0);
-		}
-		triggerEnable.setSelected(device.isTriggerEnabled());
+		// sync dialog status with device
+		updateFields();
 	}
 
 	/**
@@ -216,15 +222,6 @@ public class DeviceController extends JComponent implements ActionListener, Runn
 		// TODO: need to check if attach was successful
 		device.attach(value);
 		
-		value = (String)speedSelect.getSelectedItem();
-		int f = smartParseInt(value);
-		if (value.indexOf("M") > 0)
-			f *= 1000000;
-		else if (value.indexOf("k") > 0)
-			f *= 1000;
-		System.out.println("Rate: " + f);
-		device.setRate(f);
-		
 		value = (String)sizeSelect.getSelectedItem();
 		int s = smartParseInt(value);
 		if (value.indexOf("K") > 0)
@@ -235,14 +232,13 @@ public class DeviceController extends JComponent implements ActionListener, Runn
 		value = (String)ratioSelect.getSelectedItem();
 		double r = 0.5;
 		if (value.equals("100/0")) r = 0;
-		else if (value.equals("25/75")) r = 0.25;
+		else if (value.equals("25/75")) r = 0.75;
 		else if (value.equals("50/50")) r = 0.5;
-		else if (value.equals("75/25")) r = 0.75;
+		else if (value.equals("75/25")) r = 0.25;
 		else if (value.equals("0/100")) r = 1;
 		System.out.println("Ratio: " + r);
 		device.setRatio(r);
 
-		device.setTriggerEnabled(triggerEnable.isSelected());
 		device.setFilterEnabled(filterEnable.isSelected());
 		int m = 0;
 		int v = 0;
@@ -253,28 +249,45 @@ public class DeviceController extends JComponent implements ActionListener, Runn
 				v |= 1 << i;				
 		}
 		device.setTrigger(m, v);
+
+		int enabledChannels = 0;
+		for (int i = 0; i < channelGroup.length; i++)
+			if (channelGroup[i].isSelected())
+				enabledChannels |= 0xff << (8 * i);
+		device.setEnabledChannels(enabledChannels);
 		
 		try {
+			System.out.println("Run started");
 			capturedData = device.run();
+			System.out.println("Run completed");
+			status = DATA_READ;
 		} catch (Exception ex) {
 			// TODO: could make sense to also return half read captures if array length is corrected
 			capturedData = null;
-			ex.printStackTrace(System.out);
+			status = ABORTED;
+			System.out.println("Run aborted");
+			if (!(ex instanceof InterruptedException))
+				ex.printStackTrace(System.out);
 		}
 		device.detach();
-
-		status = DATA_READ;
 	}
 	
 	/**
-	 * Sets the enabled state of all trigger check boxes.
-	 * @param enable <code>true</code> to enable all check boxes, <code>false</code> to disable them
+	 * Sets the enabled state of all available trigger check boxes and the ratio select.
+	 * @param enable <code>true</code> to enable trigger configuration fields, <code>false</code> to disable them
 	 */
 	private void setTriggerEnabled(boolean enable) {
-		for (int i = 0; i < 32; i++) {
+		int channels = device.getAvailableChannelCount();
+		
+		for (int i = 0; i < channels; i++) {
 			triggerMask[i].setEnabled(enable);
 			triggerValue[i].setEnabled(enable);
 		}
+		for (int i = channels; i < 32; i++) {
+			triggerMask[i].setEnabled(false);
+			triggerValue[i].setEnabled(false);
+		}
+		ratioSelect.setEnabled(enable);
 	}
 	
 	/**
@@ -282,15 +295,30 @@ public class DeviceController extends JComponent implements ActionListener, Runn
 	 * @param enable <code>true</code> to enable components, <code>false</code> to disable them
 	 */
 	private void setDialogEnabled(boolean enable) {
+		triggerEnable.setEnabled(enable);
+		captureButton.setEnabled(enable);
 		portSelect.setEnabled(enable);
 		speedSelect.setEnabled(enable);
 		sizeSelect.setEnabled(enable);
-		ratioSelect.setEnabled(enable);
-		triggerEnable.setEnabled(enable);
-		if (triggerEnable.isSelected() || !enable)
-			setTriggerEnabled(enable);
+		updateFields(enable);
 	}
 
+	/** activates / deactivates dialog options according to device status */
+	public void updateFields() {
+		updateFields(true);
+	}
+
+	/** activates / deactivates dialog options according to device status */
+	private void updateFields(boolean enable) {
+		triggerEnable.setSelected(device.isTriggerEnabled());
+		if (triggerEnable.isSelected() || !enable)
+			setTriggerEnabled(enable);
+		setTriggerEnabled(device.isTriggerEnabled());
+		filterEnable.setEnabled(device.isFilterAvailable() && enable);
+		for (int i = 0; i < channelGroup.length; i++)
+			channelGroup[i].setEnabled(enable && (i < device.getAvailableChannelCount() / 8));
+	}
+	
 	/**
 	 * Properly closes the dialog.
 	 * This method makes sure timer and worker thread are stopped before the dialog is closed.
@@ -340,15 +368,18 @@ public class DeviceController extends JComponent implements ActionListener, Runn
 		} else {
 		
 			if (o == triggerEnable) {
-				boolean enable = false;
-				if (triggerEnable.isSelected())
-					enable = true;
-				setTriggerEnabled(enable);
+				device.setTriggerEnabled(triggerEnable.isSelected());
+				updateFields();
 			} else if (o == speedSelect) {
-				if (((String)speedSelect.getSelectedItem()).equals("200MHz"))
-					filterEnable.setEnabled(false);
-				else
-					filterEnable.setEnabled(true);
+				String value = (String)speedSelect.getSelectedItem();
+				int f = smartParseInt(value);
+				if (value.indexOf("M") > 0)
+					f *= 1000000;
+				else if (value.indexOf("k") > 0)
+					f *= 1000;
+				System.out.println("Rate: " + f);
+				device.setRate(f);
+				updateFields();
 			} else if (l.equals("Capture")) {
 				startCapture();
 			} else {
@@ -398,7 +429,9 @@ public class DeviceController extends JComponent implements ActionListener, Runn
 	private JCheckBox triggerEnable;
 	private JCheckBox[] triggerMask;
 	private JCheckBox[] triggerValue;
+	private JCheckBox[] channelGroup;
 	private JProgressBar progress;
+	private JButton captureButton;
 	
 	private JDialog dialog;
 	private Device device;
