@@ -27,28 +27,30 @@ import java.awt.event.WindowAdapter;
 import java.awt.event.WindowEvent;
 import java.awt.event.WindowListener;
 import java.io.File;
-import javax.swing.filechooser.FileFilter;
 
 import javax.swing.ImageIcon;
 import javax.swing.JButton;
 import javax.swing.JFileChooser;
 import javax.swing.JFrame;
+import javax.swing.JLabel;
 import javax.swing.JMenu;
 import javax.swing.JMenuBar;
 import javax.swing.JMenuItem;
+import javax.swing.JOptionPane;
 import javax.swing.JScrollPane;
 import javax.swing.JSeparator;
 import javax.swing.JToolBar;
+import javax.swing.filechooser.FileFilter;
 
 /**
  * Main frame and starter for Logic Analyzer Client.
  * <p>
  * This class only provides a simple end-user frontend and no functionality to be used by other code.
  * 
- * @version 0.3
+ * @version 0.5
  * @author Michael "Mr. Sump" Poppitz
  */
-public final class MainWindow extends WindowAdapter implements ActionListener, WindowListener {
+public final class MainWindow extends WindowAdapter implements ActionListener, WindowListener, StatusChangeListener {
 
 	/**
 	 * Creates a JMenu containing items as specified.
@@ -132,6 +134,11 @@ public final class MainWindow extends WindowAdapter implements ActionListener, W
 		JMenu diagramMenu = createMenu("Diagram", diagramEntries);
 		mb.add(diagramMenu);
 		
+		// help menu
+		String[] helpEntries = {"About"};
+		JMenu helpMenu = createMenu("Help", helpEntries);
+		mb.add(helpMenu);
+		
 		frame.setJMenuBar(mb);
 		
 		JToolBar tools = new JToolBar();
@@ -154,10 +161,14 @@ public final class MainWindow extends WindowAdapter implements ActionListener, W
 		
 		contentPane.add(tools, BorderLayout.NORTH);
 		
+		status = new JLabel(" ");
+		contentPane.add(status, BorderLayout.SOUTH);
+		
 		diagram = new Diagram();
+		diagram.addStatusChangeListener(this);
 		contentPane.add(new JScrollPane(diagram), BorderLayout.CENTER);
 
-		frame.setSize(1000, 750);
+		frame.setSize(1000, 835);
 		frame.setVisible(true);
 		frame.addWindowListener(this);
 
@@ -214,13 +225,32 @@ public final class MainWindow extends WindowAdapter implements ActionListener, W
 			
 			} else if (label.equals("Zoom Out")) {
 				diagram.zoomOut();
+
 			} else if (label.equals("Default Zoom")) {
 				diagram.zoomDefault();
-			}
+
+			} else if (label.equals("About")) {
+				JOptionPane.showMessageDialog(null,
+						"Sump's Logic Analyzer Client\n"
+						+ "\n"
+						+ "Copyright 2006 Michael Poppitz\n"
+						+ "This software is released under the GNU GPL.\n"
+						+ "\n"
+						+ "For more information see:\n"
+						+ "http://www.sump.org/projects/analyzer/",
+					"About", JOptionPane.INFORMATION_MESSAGE
+				);			}
 		} catch(Exception E) {
 			E.printStackTrace(System.out);
 		}
 	}
+
+	/** 
+	 * Handles status change requests.
+	 */
+	public void statusChanged(String s) {
+		status.setText(s);
+	}
 	
 	/**
 	 * Handles window close requests.
@@ -271,6 +301,7 @@ public final class MainWindow extends WindowAdapter implements ActionListener, W
 	private JFileChooser fileChooser;
 	private DeviceController controller;
 	private Diagram diagram;
+	private JLabel status;
 	
 	private JFrame frame;
 	private Thread thread;
